package com.tzld.piaoquan.ad.engine.commons.score.model;

import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdActionFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRankItem;
import org.apache.commons.math3.distribution.BetaDistribution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

public class VideoAdThompsonModel extends Model{
    private static final Logger LOGGER = LoggerFactory.getLogger(VideoAdThompsonModel.class);

    private Map<String, AdActionFeature> featureCache = new HashMap<>();
    private Map<Long, AdActionFeature> adActionFeatureMap = new HashMap<>();
    private Map<Long, AdActionFeature> videoAdActionFeatureMap = new HashMap<>();
    private  int alpha_click = 60;
    private int alpha_conv = 60;
    private int beta_click = 2000;
    private int beta_conversion = 2000;

    @Override
    public int getModelSize() {
        return featureCache.size();
    }

    @Override
    public boolean loadFromStream(InputStreamReader in) throws Exception {
        Map<String, AdActionFeature> initModel = new HashMap<>();
        Map<Long, AdActionFeature> tempAdActionFeatureMap =new HashMap<>();
        Map<Long, AdActionFeature> tempVideoActionFeatureMap =new HashMap<>();
        try (BufferedReader reader = new BufferedReader(in)) {
            String line = null;
            int size = 0;
            while ((line = reader.readLine()) != null) {
                String[] items = line.split(",");
                if (items.length < 5) {
                    continue;
                }
//    headvideoid   STRING
//    ,advertiserid STRING
//    ,adid         STRING
//    ,creativeid   STRING
//    ,conversion   STRING
//    ,view         STRING
//    ,click        STRING
//    ,cpa          STRING
                Long videoId=new BigDecimal(items[0].trim()).longValue();
                Long creativeId=new BigDecimal(items[4].trim()).longValue();
                double conversion=new BigDecimal(items[5].trim()).doubleValue();
                double view= new BigDecimal(items[6].trim()).doubleValue();
                double click=new BigDecimal(items[7].trim()).doubleValue();

                this.putAdFeature(tempAdActionFeatureMap,creativeId,view,click,conversion);
                this.putVideoAdFeature(tempVideoActionFeatureMap,videoId,view,click,conversion);
                this.putFeature(
                        initModel,
                        videoId,
                        creativeId,
                        view,
                        click,
                        conversion
                );
                size++;
            }
            this.featureCache = initModel;
            this.adActionFeatureMap=tempAdActionFeatureMap;
            this.videoAdActionFeatureMap=tempVideoActionFeatureMap;
            LOGGER.info("[VIDEO_CREATIVE THOMPSON MODEL LOAD] mode load over, size {}", size);
        } catch (
                Exception e) {
            LOGGER.info("[VIDEO_CREATIVE THOMPSON  MODEL LOAD] mode load error ", e);
        } finally {
            in.close();
        }

        return true;
    }
    private void putVideoAdFeature(Map<Long, AdActionFeature> cache, Long videoId , Double view, Double click, Double conversion) {
        AdActionFeature adActionFeature=cache.getOrDefault(videoId,new AdActionFeature());
        adActionFeature.setAdClick(adActionFeature.getAdClick()+click);
        adActionFeature.setAdConversion(adActionFeature.getAdConversion()+conversion);
        adActionFeature.setAdView(adActionFeature.getAdView()+view);
        cache.put(videoId,adActionFeature);
    }

    private void putAdFeature(Map<Long, AdActionFeature> cache, Long adId , Double view, Double click, Double conversion) {
        AdActionFeature adActionFeature=cache.getOrDefault(adId,new AdActionFeature());
        adActionFeature.setAdClick(adActionFeature.getAdClick()+click);
        adActionFeature.setAdConversion(adActionFeature.getAdConversion()+conversion);
        adActionFeature.setAdView(adActionFeature.getAdView()+view);
        cache.put(adId,adActionFeature);
    }

    private void putFeature(Map<String, AdActionFeature> modelCache, Long videoId, Long creativeId, Double view, Double click, Double conversion) {
        AdActionFeature adActionFeature =  new AdActionFeature();
        adActionFeature.setAdView(view);
        adActionFeature.setAdClick(click);
        adActionFeature.setAdConversion(conversion);
        modelCache.put(videoId+"_"+creativeId, adActionFeature);
    }

    public double ctrScore(AdRankItem item) {
        AdActionFeature adActionFeature = this.featureCache.getOrDefault(item.getVideoId()+"_"+item.getAdId(),  new AdActionFeature());
        if(adActionFeature.getAdView()<1000){
            adActionFeature=this.adActionFeatureMap.getOrDefault(item.getAdId(), new AdActionFeature());
        }
        int alpha_ctr = (int) adActionFeature.getAdClick() + this.alpha_click;
        int beta_ctr = this.beta_click + (int) adActionFeature.getAdView() - (int) adActionFeature.getAdClick();
        return this.betaSampler(alpha_ctr, beta_ctr);
    }

    public double cvrScore(AdRankItem item) {
        AdActionFeature adActionFeature = this.featureCache.getOrDefault(item.getVideoId()+"_"+item.getAdId(),  new AdActionFeature());
        if(adActionFeature.getAdView()<1000){
            adActionFeature=this.adActionFeatureMap.getOrDefault(item.getAdId(), new AdActionFeature());
        }
        int alpha_cvr = (int) adActionFeature.getAdConversion() + 10;
        int beta_cvr = 100 + (int) adActionFeature.getAdClick() - (int) adActionFeature.getAdConversion();
        return this.betaSampler(alpha_cvr, beta_cvr);
    }

    public double score(AdRankItem item) {
        AdActionFeature adActionFeature = this.featureCache.getOrDefault(item.getVideoId()+"_"+item.getAdId(),  new AdActionFeature());
        if(adActionFeature.getAdView()<1000){
            adActionFeature=this.adActionFeatureMap.getOrDefault(item.getAdId(), new AdActionFeature());
        }
        int alpha_cvr = (int) adActionFeature.getAdConversion() + 10;
        int beta_cvr = 100 + (int) adActionFeature.getAdClick() - (int) adActionFeature.getAdConversion();
        return this.betaSampler(alpha_cvr, beta_cvr);
    }

    public double betaSampler(double alpha, double beta) {
        BetaDistribution betaSample = new BetaDistribution(alpha, beta);
        return betaSample.sample();

    }
}
